@@ -69,8 +69,6 @@
 extern wxStfApp& wxGetApp();
 wxStfApp& wxGetApp() { return *static_cast<wxStfApp*>(wxApp::GetInstance()); }
 
-wxStfParentFrame *frame = (wxStfParentFrame *) NULL;
-
 BEGIN_EVENT_TABLE( wxStfApp, wxApp )
 EVT_KEY_DOWN( wxStfApp::OnKeyDown )
 
@@ -162,13 +160,13 @@ bool wxStfApp::OnInit(void)
     //// Create a document manager
     wxDocManager* docManager = new wxDocManager;
     //// Create a template relating drawing documents to their views
-#if (defined(WITH_BIOSIG) || defined(WITH_BIOSIG2))
+#if defined(WITH_BIOSIG)
     m_biosigTemplate=new wxDocTemplate( docManager,
                                      wxT("All files"), wxT("*.*"), wxT(""), wxT(""),
                                      wxT("Biosig Document"), wxT("Biosig View"), CLASSINFO(wxStfDoc),
                                      CLASSINFO(wxStfView) );
 #endif
-#if (defined(WITH_BIOSIG) || defined(WITH_BIOSIG2))
+#if defined(WITH_BIOSIG)
     m_biosigTemplate=new wxDocTemplate( docManager,
                                      wxT("Biosig files"), wxT("*.dat;*.cfs;*.gdf;*.ibw;*.wcp"), wxT(""), wxT(""),
                                      wxT("Biosig Document"), wxT("Biosig View"), CLASSINFO(wxStfDoc),
@@ -189,11 +187,7 @@ bool wxStfApp::OnInit(void)
                                      wxT("ABF Document"), wxT("ABF View"), CLASSINFO(wxStfDoc),
                                      CLASSINFO(wxStfView) );
 #if defined(__WXGTK__) || defined(__WXMAC__)
-#if !defined(__MINGW32__)
-#if !defined(WITHOUT_ABF)
     ABF_Initialize();
-#endif
-#endif
 #endif
     m_atfTemplate=new wxDocTemplate( docManager,
                                      wxT("Axon text file"), wxT("*.atf"), wxT(""), wxT("atf"),
@@ -235,7 +229,7 @@ bool wxStfApp::OnInit(void)
     docManager->SetLastDirectory( lastDir );
 
     //// Create the main frame window
-    frame = new wxStfParentFrame(docManager, (wxFrame *)NULL,
+    wxStfParentFrame* frame = new wxStfParentFrame(docManager, nullptr,
                                  wxT("Stimfit"), wxDefaultPosition,
 #ifndef __WXMAC__
                                  wxSize(1024, 768),
@@ -323,7 +317,7 @@ bool wxStfApp::OnInit(void)
     
     // check for updates in background:
 #ifndef __WXMAC__
-    frame->CheckUpdate();
+    frame->CheckUpdate(NULL);
 #endif
     // load user-defined plugins:
     // pluginLib = stf::GetPluginLib();
@@ -873,9 +867,9 @@ wxStfChildFrame *wxStfApp::CreateChildFrame(wxDocument *doc, wxView *view)
 #endif
     wxStfChildFrame *subframe = new wxStfChildFrame(
                                                     doc, view, 
-                                                    GetMainFrame(), wxID_ANY, doc->GetTitle(),
+                                                    wxStaticCast(GetTopWindow(), wxStfParentFrame), wxID_ANY, doc->GetTitle(),
 #ifdef __WXMAC__
-                                                    wxPoint(xpos,ypos), wxSize(800,600),
+                                                    wxDefaultPosition, wxDefaultSize,
 #else
                                                     wxDefaultPosition, wxDefaultSize,
 #endif
@@ -981,7 +975,7 @@ void wxStfApp::OnKeyDown( wxKeyEvent& event ) {
 void wxStfApp::OnCursorSettings( wxCommandEvent& WXUNUSED(event) ) {
     wxStfDoc* actDoc=GetActiveDoc();
     if (CursorsDialog==NULL && actDoc!=NULL) {
-        CursorsDialog=new wxStfCursorsDlg(frame, actDoc);
+        CursorsDialog=new wxStfCursorsDlg((wxStfParentFrame*)GetTopWindow(), actDoc);
         CursorsDialog->Show();
         CursorsDialog->SetActiveDoc(actDoc);
         //set CEdit controls to given values
@@ -1259,7 +1253,7 @@ bool wxStfApp::OpenFileSeries(const wxArrayString& fNameArray) {
     if (nFiles!=1) {
         // Ask whether to put files into a single window:
         singleWindow=(wxMessageDialog(
-                                      frame,
+                                      (wxStfParentFrame*)GetTopWindow(),
                                       wxT("Put files into a single window?"),
                                       wxT("File series import"),
                                       wxYES_NO
@@ -1269,7 +1263,7 @@ bool wxStfApp::OpenFileSeries(const wxArrayString& fNameArray) {
                              wxT("Importing file series"),
                              wxT("Starting file import"),
                              100,
-                             frame,
+                             (wxStfParentFrame*)GetTopWindow(),
                              wxPD_SMOOTH | wxPD_AUTO_HIDE
                              );
     int n_opened=0;
@@ -1439,11 +1433,6 @@ wxString wxStfApp::GetVersionString() const {
     return verString;
 }
 
-StfDll wxStfParentFrame *GetMainFrame(void)
-{
-    return frame;
-}
-
 //  LocalWords:  wxStfView
 
 #ifdef WITH_PYTHON
@@ -1452,7 +1441,7 @@ void wxStfApp::OnPythonImport(wxCommandEvent& WXUNUSED(event)) {
     // show a file selection dialog menu.
     wxString pyFilter; // file filter only show *.py
     pyFilter = wxT("Python file (*.py)|*.py");
-    wxFileDialog LoadModuleDialog (frame,
+    wxFileDialog LoadModuleDialog ((wxStfParentFrame*)GetTopWindow(),
                 wxT("Import/reload Python module"),
                 wxT(""),
                 wxT(""),
